VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cLevenshtein"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'  cLevenshtein
' --------------
' Berechnet die Levenshtein-Distanz zwischen zwei Strings.
' s. auch
'   Algemeines zum Algorithmus:    http://de.wikipedia.org/wiki/Levenstein-Distanz
'   Visual-Basic Sourcecode:       http://www.merriampark.com/ld.htm#VB
'
' Entwickelt fr: ActiveVB [http://www.acticvevb.de]
'
' Autor(en):            Peter Krner
' Zuletzt bearbeitet:   10.06.2005
' Version: 1.2
' Changelog:
'   v1.0    Peter Krner
'       Erstes Release
'   v1.1    Peter Krner
'       Property CaseSensitive hinzugefgt
'   v1.2    Peter Krner
'       Kleinere Bugs beseitigt, Hauptroutine
'       auf Long umgestellt
'

Private m_StringOne As String
Private m_StringTwo As String
Private m_Similarity As Integer
Private m_CaseSensitive As Boolean
Private m_Distance As Integer

Public Function Compare( _
        Optional ByVal StringOne As String, _
        Optional ByVal StringTwo As String, _
        Optional ByVal CaseSensitive As Boolean) As Integer
    
    Dim D() As Integer
    Dim m As Long, n As Long
    Dim i As Long, j As Long
    Dim min As Long, max As Long
    Dim cost As Byte
    
    m_StringOne = StringOne
    m_StringTwo = StringTwo
    m_CaseSensitive = CaseSensitive
    
    If Not m_CaseSensitive Then
        StringOne = UCase(StringOne)
        StringTwo = UCase(StringTwo)
    End If
    
    n = Len(StringOne)
    m = Len(StringTwo)
    
    If n = 0 Then
        m_Distance = m
    ElseIf m = 0 Then
        m_Distance = n
    Else
        ReDim D(0 To n, 0 To m)
        For i = 0 To n
            D(i, 0) = i
        Next i
        For j = 0 To m
            D(0, j) = j
        Next j
        
        For i = 1 To n
            For j = 1 To m
                If Mid$(StringOne, i, 1) = Mid$(StringTwo, j, 1) Then cost = 0 Else cost = 1
                D(i, j) = Minimum( _
                    D(i - 1, j) + 1, _
                    D(i, j - 1) + 1, _
                    D(i - 1, j - 1) + cost _
                )
            Next j
        Next i
        
        m_Distance = D(n, m)
        Erase D
    End If
    
    Compare = m_Distance
    If m > n Then max = m Else max = n
    If m_Distance = max Then
        m_Similarity = 0
    ElseIf m_Distance = min Then
        m_Similarity = 100
    Else
        m_Similarity = 100 - m_Distance / max * 100
    End If
    Debug.Print m_Similarity
End Function

Public Property Get StringOne() As String
    StringOne = m_StringOne
End Property
Public Property Let StringOne(NewValue As String)
    m_StringOne = NewValue
    Call Me.Compare
End Property

Public Property Get StringTwo() As String
    StringTwo = m_StringTwo
End Property
Public Property Let StringTwo(NewValue As String)
    m_StringTwo = NewValue
    Call Me.Compare
End Property

Public Property Get CaseSensitive() As Boolean
    CaseSensitive = m_CaseSensitive
End Property
Public Property Let CaseSensitive(NewValue As Boolean)
    m_CaseSensitive = NewValue
End Property


Public Property Get Similarity() As Integer
    Similarity = m_Similarity
End Property

Public Property Get Distance() As Integer
    Distance = m_Distance
End Property

Private Function Minimum(ByVal a As Integer, ByVal b As Integer, ByVal c As Integer) As Integer
    Dim mi As Integer
    
    mi = a
    If b < mi Then
        mi = b
    End If
    If c < mi Then
        mi = c
    End If
    Minimum = mi
End Function
